# File post-extraction formatters for musdex
#
# Copyright 2010 Max Battcher. Some rights reserved.
# Licensed for use under the Ms-RL. See attached LICENSE file.
import subprocess

def xmllint(filename):
    subprocess.call(['xmllint', '--format', '--output', filename, filename])

def remove_carriage_returns(filename):
    import os
    bakfile = "%s.bak~" % filename
    os.rename(filename, bakfile)
    inf = open(bakfile, 'rb')
    outf = open(filename, 'wb')
    for line in inf:
        outf.write(line.replace('\r', ''))
    inf.close()
    outf.close()
    os.remove(bakfile)

_fmt_cache = {'xmllint': xmllint, 'removecrs': remove_carriage_returns}

def get_formatter(name=None):
    if name is None:
        return
    elif name not in _fmt_cache:
        pieces = name.rsplit('.', 1)
        _temp = __import__(pieces[0],
            globals(),
            locals(),
            [pieces[1]],
            -1,
        )
        _fmt_cache[name] = getattr(_temp, pieces[1])
    return _fmt_cache[name]

# vim: ai et ts=4 sts=4 sw=4



# VCS routines for musdex
#
# Copyright 2010 Max Battcher. Some rights reserved.
# Licensed for use under the Ms-RL. See attached LICENSE file.
from subprocess import CalledProcessError, PIPE, Popen, check_call
import logging
import os.path

DARCS_ADD = 'darcs add'
DARCS_REMOVE = 'darcs remove'
DARCS_SHOW_FILES = 'darcs show files --no-directories'

def manifest(config):
    logging.debug("Loading manifest")
    cmd = config["vcs_show_files"] if "vcs_show_files" in config \
        else DARCS_SHOW_FILES
    cmd = cmd.split(' ')
    pr = Popen(cmd, stdout=PIPE)
    manifest = pr.communicate()[0]
    if pr.returncode != 0:
        raise CalledProcessError(pr.returncode, cmd[0])

    # ASSUME: Broken by newlines with no filenames with newlines
    files = [os.path.relpath(f.strip()) for f in manifest.split('\n')
        if f.strip()]
    return files

def add_file(config, file):
    logging.debug("Adding %s" % file)
    cmd = config["vcs_add"] if "vcs_add" in config else DARCS_ADD
    cmd = cmd.split(' ')
    cmd.append(file)
    check_call(cmd)

def remove_file(config, file):
    logging.debug("Removing %s" % file)
    cmd = config["vcs_remove"] if "vcs_remove" in config else DARCS_REMOVE
    cmd = cmd.split(' ')
    cmd.append(file)
    check_call(cmd)

# vim: ai et ts=4 sts=4 sw=4



# File-type handlers for musdex
#
# Copyright 2010 Max Battcher. Some rights reserved.
# Licensed for use under the Ms-RL. See attached LICENSE file.
import datetime
import logging
import os.path
import time
import zipfile

def _datetime_from_epoch(epoch):
    return datetime.datetime(*time.localtime(epoch)[:6])

class ZipArchiveHandler(object):
    def __init__(self, archive, location, manifest={}):
        self.archive = archive
        self.location = location
        self.manifest = manifest

    def check(self):
        return zipfile.is_zipfile(self.archive)

    def extract(self, force=False):
        manifestfiles = set(self.manifest.keys())
        if force:
            logging.info("Extracting all of %s" % self.archive)
            ziparchive = zipfile.ZipFile(self.archive)
            ziparchive.extractall(self.location)

            for info in ziparchive.infolist():
                path = os.path.relpath(os.path.join(self.location,
                    info.filename))
                yield (path, datetime.datetime(*info.date_time))
                if path in manifestfiles: manifestfiles.remove(path)
        else:
            logging.info("Selectively extracting %s" % self.archive)
            ziparchive = zipfile.ZipFile(self.archive)

            for info in ziparchive.infolist():
                path = os.path.relpath(os.path.join(self.location,
                    info.filename))
                time = datetime.datetime(*info.date_time)
                if path not in self.manifest:
                    logging.debug("Extracting new file %s" % path)
                    ziparchive.extract(info, self.location)
                    yield (path, time)
                elif time > self.manifest[path]:
                    logging.debug("Extracting updated file %s" % path)
                    ziparchive.extract(info, self.location)
                    yield (path, time)
                if path in manifestfiles: manifestfiles.remove(path)

        # Check for removed files
        if manifestfiles:
            for f in manifestfiles:
                yield (f, None)

        yield (self.location,
            _datetime_from_epoch(os.path.getmtime(self.archive)))

    def combine(self, force=False):           
        # We can ignore force here, because zipfile doesn't have strong
        # support for incremental file updates so we are *always*
        # re-combining the entire file.
        logging.info("Combining %s" % self.archive)
        ziparchive = zipfile.ZipFile(self.archive, 'w',
            zipfile.ZIP_DEFLATED)
        for file in self.manifest:
            ziparchive.write(file, os.path.relpath(file, self.location))
            yield (file, _datetime_from_epoch(os.path.getmtime(file)))
        ziparchive.close()
        yield (self.location, 
            _datetime_from_epoch(os.path.getmtime(self.archive)))

_handler_cache = {}

def get_handler(handler_name=None):
    if handler_name is None:
        return ZipArchiveHandler
    elif handler_name not in _handler_cache:
        pieces = handler_name.rsplit('.', 1)
        _temp = __import__(pieces[0],
            globals(),
            locals(),
            [pieces[1]],
            -1,
        )
        _handler_cache[handler_name] = getattr(_temp, pieces[1])
    return _handler_cache[handler_name]

# vim: ai et ts=4 sts=4 sw=4



#!/usr/bin/python
from distutils.core import setup

setup(
    name='musdex',
    version='10.07.20',
    description='musdex -- VCS-aware zip archive tool',
    author='Max Battcher',
    author_email='me@worldmaker.net',
    url='http://musdex.code.worldmaker.net',
    packages=['musdex'],
    scripts=['scripts/musdex', 'scripts/xedsum'],
    requires=[
        'argparse',
        'yaml',
    ],
    license='Microsoft Reciprocal License (Ms-RL)',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Topic :: Software Development :: Version Control',
    ],
)

# vim: ai et ts=4 sts=4 sw=4
